
#include <boost/shared_ptr.hpp>
#include <iostream>


using boost::shared_ptr;
using std::cout;
using std::endl;


template<typename Key>
struct node
{
	Key key;
	
	shared_ptr<node<Key> > left;
	
	shared_ptr<node<Key> > right;
	
	shared_ptr<node<Key> > parent;
	
	enum Color {RED, BLACK} color;

	node() : color(RED)
	{
		cout << "node::node():" << endl;
	}
	
	node(Key a_key) : key(a_key), color(RED)
	{
		cout << "node::node(): a_key=" << a_key << endl;
	}
	
	~node()
	{
		cout << "node::~node(): key=" << key << ", right.use_count()=" << right.use_count() << endl;
	}

	bool is_sentinel()
	{
		return color == BLACK && left.get() == 0 && right.get() == 0;
	}

	static shared_ptr<node<Key> > create_node(Key key)
	{
		shared_ptr<node<Key> > new_node(new node<Key>(key));
		new_node->left = create_sentinel(new_node);
		new_node->right = create_sentinel(new_node);
		return new_node;
	}

	static shared_ptr<node<Key> > create_sentinel(shared_ptr<node<Key> > parent)
	{
		shared_ptr<node<Key> > new_node(new node<Key>);
		new_node->color = BLACK;
		new_node->parent = parent;
		return new_node;
	}
};


template<typename Key>
class rb_tree
{
public:

	explicit rb_tree();

	~rb_tree();

	shared_ptr<node<Key> > find(Key key);

	void insert(Key key);

	void remove(Key key);

private:

	shared_ptr<node<Key> > predecessor(shared_ptr<node<Key> > a_node);

	shared_ptr<node<Key> > successor(shared_ptr<node<Key> > a_node);

	void rotate_left(shared_ptr<node<Key> > a_node);

	void rotate_right(shared_ptr<node<Key> > a_node);

	void destroy_subtree(shared_ptr<node<Key> > root);

	rb_tree(const rb_tree&);

	void operator=(const rb_tree&);

	shared_ptr<node<Key> > _root;
};


